#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==================================================================== #
# ssh readonly@prodiguer-test-db.ipsl.upmc.fr                          #
# psql -U prodiguer_db_user prodiguer                                  #
#                                                                      #
# ssh readonly@prodiguer-test-db.ipsl.upmc.fr -L 5432:localhost:5432   #
# ==================================================================== #


# This must come first
from __future__ import print_function, unicode_literals, division

# Standard library imports
import os
import math
# import datetime as dt
from argparse import ArgumentParser

# Application library imports
import libconsodb as cdb


########################################
def get_arguments():
  parser = ArgumentParser()
  parser.add_argument("project", action="store",
                      help="Project name")
  parser.add_argument("center", action="store",
                      help="Center name (idris/tgcc)")

  parser.add_argument("-v", "--verbose", action="store_true",
                      help="verbose mode")
  parser.add_argument("-d", "--dryrun", action="store_true",
                      help="only print what is to be done")
  parser.add_argument("-r", "--range", action="store", nargs=2,
                      help="date range: ssaammjj ssaammjj")

  # parser.add_argument("-f", "--full", action="store_true",
  #                     help="plot the whole period")
  # parser.add_argument("-i", "--increment", action="store",
  #                     type=int, default=1, dest="inc",
  #                     help="sampling increment")
  # parser.add_argument("-m", "--max", action="store_true",
  #                     help="plot with y_max = allocation")
  # parser.add_argument("-s", "--show", action="store_true",
  #                     help="interactive mode")
  # parser.add_argument("-d", "--dods", action="store_true",
  #                     help="copy output on dods")

  return parser.parse_args()


########################################
if __name__ == "__main__":

  # .. Initialization ..
  # ====================
  # ... Command line arguments ...
  # ------------------------------
  args = get_arguments()
  if args.verbose:
    print(args)

  ROOT_DIR = "/home_local/slipsl/ConsoGENCMIP6/init_db"
  DATA_DIR = os.path.join(ROOT_DIR, "data")

  SUBMIT_DIR = os.getcwd()

  # project = "rgzi"
  # centre = "idris"

  conso_per_login = {}

  # Connection info
  db_host = "134.157.170.104"
  # db_port = "5432"
  db_name = "prodiguer"
  db_user = "prodiguer_db_user"
  # db_pwd = "secret"

  if args.center == "idris":
    pattern = "compta_*.dat"
  elif args.center == "tgcc":
    pattern = "ccc_myproject.dat_*"
  else:
    print("Unknown center {}".format(args.center))
    exit()

  filelist = cdb.find_input_files(DATA_DIR, pattern, args.range)

  # Build dictionary from files
  # for filename in cdb.find_idris_files(DATA_DIR):
  for filename in filelist:
    print(filename)
    # jour, heure, alloc, consos = \
    date, alloc, consos = \
          cdb.parse_input_cpt(filename, args.project, args.center)
          # cdb.parse_idris_cpt(filename, args.project)

    # date = dt.datetime.strptime("{} {}".format(jour, heure), "%d/%m/%Y %H:%M")

    for item in consos:
      login, conso = item
      if login not in conso_per_login:
        conso_per_login[login] = set()
      conso_per_login[login].add((date, conso, ))

  conn, cursor = cdb.connect_db(db_host, db_name, db_user)

  # Extract allocation id from table
  table_name = "conso.tbl_allocation"
  request = (
    "SELECT id "
    "FROM " + table_name + " "
    "WHERE project = '" + args.project + "'"
    "  AND centre = '" + args.center + "'"
    ";"
  )

  cdb.select_db(cursor, request)
  if cursor.rowcount != 1:
    print(cursor.fetchall())
    exit()

  (allocation_id, ) = cursor.fetchone()

  for login, conso_list in conso_per_login.iteritems():

    # test = "'"+login+"'" if login != "total" else "NULL"
    # print(test)

    lines_req = [
      (
        "('{alloc}', "
        "'{date}', "
        "{total_hrs}, "
        "{login}, "
        "{create})"
      ) .format(
        alloc=allocation_id,
        date=date,
        total_hrs=conso if not math.isnan(conso) else "'NaN'",
        login="'"+login+"'" if login != "total" else "NULL",
        create="CURRENT_TIMESTAMP",
      ) for (date, conso) in conso_list
    ]

    table_name = "conso.tbl_consumption"
    request = (
      "INSERT INTO " + table_name + " ("
      "  allocation_id, "
      "  date, "
      "  total_hrs, "
      "  login, "
      "  row_create_date "
      ") "
      "VALUES "
    )
    request = request + ", ".join(lines_req)
    print(request)

    cdb.insert_db(cursor, request)

    if not args.dryrun:
      cdb.commit_db(conn)
